@@ -48,11 +48,7 @@
 #include <qstyleoption.h>
 #include <qapplication.h>
 #include <qquickwindow.h>
-#if QT_VERSION >= QT_VERSION_CHECK(5,8,0)
 #include <QtQuick/qsgninepatchnode.h>
-#else
-#include "qsgdefaultninepatchnode_p.h"
-#endif
 
 #include <Kirigami2/PlatformTheme>
 
@@ -86,39 +82,39 @@ KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
     setSmooth(false);
     qmlRegisterType<KQuickPadding>();
 
-    connect(this, SIGNAL(visibleChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(widthChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(heightChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(enabledChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(infoChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(onChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(selectedChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(activeChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(textChanged()), this, SLOT(updateSizeHint()));
-    connect(this, SIGNAL(textChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(activeChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(raisedChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(sunkenChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(hoverChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(maximumChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(minimumChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(valueChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(horizontalChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(transientChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(activeControlChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(hasFocusChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(activeControlChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(hintChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(propertiesChanged()), this, SLOT(updateSizeHint()));
-    connect(this, SIGNAL(propertiesChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(elementTypeChanged()), this, SLOT(updateItem()));
-    connect(this, SIGNAL(contentWidthChanged(int)), this, SLOT(updateSizeHint()));
-    connect(this, SIGNAL(contentHeightChanged(int)), this, SLOT(updateSizeHint()));
-    connect(this, SIGNAL(widthChanged()), this, SLOT(updateRect()));
-    connect(this, SIGNAL(heightChanged()), this, SLOT(updateRect()));
+    connect(this, &KQuickStyleItem::visibleChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::widthChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::heightChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::enabledChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::infoChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::onChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::selectedChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::activeChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::textChanged, this, &KQuickStyleItem::updateSizeHint);
+    connect(this, &KQuickStyleItem::textChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::activeChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::raisedChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::sunkenChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::hoverChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::maximumChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::minimumChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::valueChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::horizontalChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::transientChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::activeControlChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::hasFocusChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::activeControlChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::hintChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::propertiesChanged, this, &KQuickStyleItem::updateSizeHint);
+    connect(this, &KQuickStyleItem::propertiesChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::elementTypeChanged, this, &KQuickStyleItem::updateItem);
+    connect(this, &KQuickStyleItem::contentWidthChanged, this, &KQuickStyleItem::updateSizeHint);
+    connect(this, &KQuickStyleItem::contentHeightChanged, this, &KQuickStyleItem::updateSizeHint);
+    connect(this, &KQuickStyleItem::widthChanged, this, &KQuickStyleItem::updateRect);
+    connect(this, &KQuickStyleItem::heightChanged, this, &KQuickStyleItem::updateRect);
 
-    connect(this, SIGNAL(heightChanged()), this, SLOT(updateBaselineOffset()));
-    connect(this, SIGNAL(contentHeightChanged(int)), this, SLOT(updateBaselineOffset()));
+    connect(this, &KQuickStyleItem::heightChanged, this, &KQuickStyleItem::updateBaselineOffset);
+    connect(this, &KQuickStyleItem::contentHeightChanged, this, &KQuickStyleItem::updateBaselineOffset);
 }
 
 KQuickStyleItem::~KQuickStyleItem()
@@ -716,7 +712,16 @@ void KQuickStyleItem::resolvePalette()
     if (QCoreApplication::testAttribute(Qt::AA_SetPalette))
         return;
 
+    const QVariant controlPalette = m_control ? m_control->property("palette") : QVariant();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
+    if (controlPalette.isValid()) {
+        m_styleoption->palette = controlPalette.value<QPalette>();
+    } else {
+        m_styleoption->palette = m_theme->palette();
+    }
+#else
     m_styleoption->palette = m_theme->palette();
+#endif
 }
 
 /*
@@ -923,7 +928,8 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
         break;
     default:
         break;
-    }    return size;
+    }
+    return size.expandedTo(QSize(m_contentWidth, m_contentHeight));
 }
 
 qreal KQuickStyleItem::baselineOffset()
@@ -1009,7 +1015,6 @@ void KQuickStyleItem::updateRect()
 
 int KQuickStyleItem::pixelMetric(const QString &metric)
 {
-
     if (metric == QLatin1String("scrollbarExtent"))
         return qApp->style()->pixelMetric(QStyle::PM_ScrollBarExtent, nullptr);
     else if (metric == QLatin1String("defaultframewidth"))
@@ -1052,6 +1057,18 @@ int KQuickStyleItem::pixelMetric(const QString &metric)
         return abs(qApp->style()->pixelMetric(QStyle::PM_ScrollView_ScrollBarSpacing, nullptr));
     else if (metric == QLatin1String("treeviewindentation"))
         return qApp->style()->pixelMetric(QStyle::PM_TreeViewIndentation, nullptr);
+    else if (metric == QLatin1String("layouthorizontalspacing"))
+        return qApp->style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing, nullptr);
+    else if (metric == QLatin1String("layoutverticalspacing"))
+        return qApp->style()->pixelMetric(QStyle::PM_LayoutVerticalSpacing, nullptr);
+    else if (metric == QLatin1String("layoutleftmargin"))
+        return qApp->style()->pixelMetric(QStyle::PM_LayoutLeftMargin, nullptr);
+    else if (metric == QLatin1String("layouttopmargin"))
+        return qApp->style()->pixelMetric(QStyle::PM_LayoutTopMargin, nullptr);
+    else if (metric == QLatin1String("layoutrightmargin"))
+        return qApp->style()->pixelMetric(QStyle::PM_LayoutRightMargin, nullptr);
+    else if (metric == QLatin1String("layoutbottommargin"))
+        return qApp->style()->pixelMetric(QStyle::PM_LayoutBottomMargin, nullptr);
     return 0;
 }
 
@@ -1555,10 +1572,29 @@ void KQuickStyleItem::setControl(QQuickItem *control)
 
     if (m_control) {
         m_control->removeEventFilter(this);
+        disconnect(m_control, 0, this, 0);
     }
 
     m_control = control;
-    m_control->installEventFilter(this);
+
+    if (m_control) {
+        m_control->installEventFilter(this);
+    
+        if (m_control->window()) {
+            m_window = m_control->window();
+            m_window->installEventFilter(this);
+        }
+        connect(m_control, &QQuickItem::windowChanged, this,
+                [this](QQuickWindow *window) {
+            if (m_window) {
+                m_window->removeEventFilter(this);
+            }
+            m_window = window;
+            if (m_window) {
+                m_window->installEventFilter(this);
+            }
+        });
+    }
 
     emit controlChanged();
 }
@@ -1572,11 +1608,7 @@ QSGNode *KQuickStyleItem::updatePaintNode(QSGNode *node, UpdatePaintNodeData *)
 
     QSGNinePatchNode *styleNode = static_cast<QSGNinePatchNode *>(node);
     if (!styleNode)
-#if QT_VERSION >= QT_VERSION_CHECK(5,8,0)
         styleNode = window()->createNinePatchNode();
-#else
-        styleNode = new QSGDefaultNinePatchNode;
-#endif
 
 #ifdef QSG_RUNTIME_DESCRIPTION
     qsgnode_set_description(styleNode,
@@ -1616,9 +1648,19 @@ void KQuickStyleItem::updatePolish()
 
 bool KQuickStyleItem::eventFilter(QObject *watched, QEvent *event)
 {
-    if (event->type() == QEvent::FocusIn || event->type() == QEvent::FocusOut) {
-        QFocusEvent *fe = static_cast<QFocusEvent *>(event);
-        m_lastFocusReason = fe->reason();
+    if (watched == m_control) {
+        if (event->type() == QEvent::FocusIn || event->type() == QEvent::FocusOut) {
+            QFocusEvent *fe = static_cast<QFocusEvent *>(event);
+            m_lastFocusReason = fe->reason();
+        }
+    } else if (watched == m_window.data()) {
+        if (event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
+            QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+            if (ke->key() == Qt::Key_Alt) {
+                updateItem();
+            }
+
+        }
     }
 
     return QQuickItem::eventFilter(watched, event);
